
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>

//#include "cnc/cnc-interface.h"
#include "heightmap.h"

float heightmap_check_max_z(int width, int height) {
	int x, y;
	float z_data[1];
	float z_max = 0;
	for (y = height - 1; y >= 0; y--) {
		for (x = 0; x < width; x++) {
			glReadPixels(x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
			if (z_data[0] > z_max) {
				z_max = z_data[0];
			}
		}
	}
	return z_max;
}

float heightmap_check_min_z(int width, int height) {
	int x, y;
	float z_data[1];
	float z_min = 100000;
	for (y = height - 1; y >= 0; y--) {
		for (x = 0; x < width; x++) {
			glReadPixels(x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
			if (z_data[0] < z_min) {
				z_min = z_data[0];
			}
		}
	}
	return z_min;
}

float cnc_glcheck_get_pixel_depth(int x, int y) {
	static float z_data[1];
	glReadPixels(x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
	return z_data[0];
}

float cnc_glcheck_get_circle_depth(int x0, int y0, int radius1, int radius2) {
	int f;
	int ddF_x;
	int ddF_y;
	int x;
	int y;
	float z;
	int radius;
	float min_z = 100000;
	for (radius = radius2; radius >= radius1; radius--) {
		f = 1 - radius;
		ddF_x = 0;
		ddF_y = -2 * radius;
		x = 0;
		y = radius;
		if ((z = cnc_glcheck_get_pixel_depth(x0, y0 + radius)) < min_z) {
			min_z = z;
		}
		if ((z = cnc_glcheck_get_pixel_depth(x0, y0 - radius)) < min_z) {
			min_z = z;
		}
		if ((z = cnc_glcheck_get_pixel_depth(x0 + radius, y0)) < min_z) {
			min_z = z;
		}
		if ((z = cnc_glcheck_get_pixel_depth(x0 - radius, y0)) < min_z) {
			min_z = z;
		}
		while(x < y) {
			if(f >= 0) {
				y--;
				ddF_y += 2;
				f += ddF_y;
			}
			x++;
			ddF_x += 2;
			f += ddF_x + 1;
			if ((z = cnc_glcheck_get_pixel_depth(x0 + x, y0 + y)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 - x, y0 + y)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 + x, y0 - y)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 - x, y0 - y)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 + y, y0 + x)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 - y, y0 + x)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 + y, y0 - x)) < min_z) {
				min_z = z;
			}
			if ((z = cnc_glcheck_get_pixel_depth(x0 - y, y0 - x)) < min_z) {
				min_z = z;
			}
		}
	}
	return min_z;
}

int heightmap_export_tga(const char *filename, int width, int height) {
	FILE *f = NULL;
	int x, y, z;
	float z_data[1];
	if ((f = fopen( filename, "w" )) != NULL) {
		fputc (0x00, f);	/* ID Length, 0 => No ID	*/
		fputc (0x00, f);	/* Color Map Type, 0 => No color map included	*/
		fputc (0x02, f);	/* Image Type, 2 => Uncompressed, True-color Image */
		fputc (0x00, f);	/* Next five bytes are about the color map entries */
		fputc (0x00, f);	/* 2 bytes Index, 2 bytes length, 1 byte size */
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);	/* X-origin of Image	*/
		fputc (0x00, f);
		fputc (0x00, f);	/* Y-origin of Image	*/
		fputc (0x00, f);
		fputc (width & 0xff, f);      /* Image Width	*/
		fputc ((width>>8) & 0xff, f);
		fputc (height & 0xff, f);     /* Image Height	*/
		fputc ((height>>8) & 0xff, f);
		fputc (0x18, f);		/* Pixel Depth, 0x18 => 24 Bits	*/
		fputc (0x20, f);		/* Image Descriptor	*/
		fclose(f);
		f = fopen( filename, "ab" );  /* reopen in binary append mode */
		for (y = height - 1; y >= 0; y--) {
			//fprintf(stderr, "%i\n", 100 - (100 / height * y));
			for (x = 0; x < width; x++) {
				glReadPixels(x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
				z = (int)(z_data[0] * 255);
				fputc(z, f);
				fputc(z, f);
				fputc(z, f);
			}
		}
		fclose(f);
	}
	return 0;
}



