
#include <stdio.h>
#include <stdlib.h>

#include "osmesa.h"

int osmesa_init(int width, int height) {
	osmesa_ctx = OSMesaCreateContextExt(GL_RGBA, 16, 0, 0, NULL);
	if (!osmesa_ctx) {
		fprintf(stderr, "OSMesaCreateContext failed!\n");
		return 1;
	}
	osmesa_buffer = (GLushort *) malloc( width * height * 4 * sizeof(GLushort));
	if (!osmesa_buffer) {
		fprintf(stderr, "Alloc image osmesa_buffer failed!\n");
		return 1;
	}
	if (!OSMesaMakeCurrent(osmesa_ctx, osmesa_buffer, GL_UNSIGNED_SHORT, width, height)) {
		fprintf(stderr, "OSMesaMakeCurrent failed!\n");
		return 1;
	}
	glEnable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-12.5, 12.5, -12.5, 12.5, -12.0, 12.0);
	glMatrixMode(GL_MODELVIEW);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glColor4f(0.5, 0.5, 0.5, 1.0);
	return 0;
}

int osmesa_exit(void) {
	free(osmesa_buffer);
	OSMesaDestroyContext(osmesa_ctx);
	return 0;
}

int osmesa_save_targa(const char *filename, const GLushort *buffer, int width, int height) {
	FILE *f = fopen( filename, "w" );
	if (f != NULL) {
		int i, x, y;
		const GLushort *ptr = buffer;
		fputc (0x00, f);	/* ID Length, 0 => No ID	*/
		fputc (0x00, f);	/* Color Map Type, 0 => No color map included	*/
		fputc (0x02, f);	/* Image Type, 2 => Uncompressed, True-color Image */
		fputc (0x00, f);	/* Next five bytes are about the color map entries */
		fputc (0x00, f);	/* 2 bytes Index, 2 bytes length, 1 byte size */
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);	/* X-origin of Image	*/
		fputc (0x00, f);
		fputc (0x00, f);	/* Y-origin of Image	*/
		fputc (0x00, f);
		fputc (width & 0xff, f);      /* Image Width	*/
		fputc ((width>>8) & 0xff, f);
		fputc (height & 0xff, f);     /* Image Height	*/
		fputc ((height>>8) & 0xff, f);
		fputc (0x18, f);		/* Pixel Depth, 0x18 => 24 Bits	*/
		fputc (0x20, f);		/* Image Descriptor	*/
		fclose(f);
		f = fopen( filename, "ab" );  /* reopen in binary append mode */
		for (y=height-1; y>=0; y--) {
			for (x=0; x<width; x++) {
				i = (y*width + x) * 4;
				fputc(ptr[i+2] >> 8, f); /* write blue */
				fputc(ptr[i+1] >> 8, f); /* write green */
				fputc(ptr[i] >> 8, f);   /* write red */
			}
		}
	}
	return 0;
}

